from logging import INFO, LogRecord
from typing import cast

import pytest

from sag_py_logging import ConsoleExtraFieldFilter, ExtraFieldsLogRecord


@pytest.fixture()
def log_record() -> LogRecord:
    return LogRecord(name="", level=INFO, pathname="", lineno=0, msg="Hello, world!", args=(), exc_info=None)


def test_without_extra_fields(log_record: LogRecord) -> None:
    # Arrange
    filter_ = ConsoleExtraFieldFilter()

    # Act
    filter_.filter(log_record)

    # Assert
    assert cast(ExtraFieldsLogRecord, log_record).stringified_extra == ""


def test_with_extra_fields(log_record: LogRecord) -> None:
    # Arrange
    filter_ = ConsoleExtraFieldFilter()
    log_record.my_extra_string = "test"
    log_record.my_extra_int = 1
    log_record.my_extra_object = {"keyOne": "valueOne", "keyTwo": 2}

    # Act
    filter_.filter(log_record)

    # Assert
    assert (
        cast(ExtraFieldsLogRecord, log_record).stringified_extra
        == 'my_extra_string="test", my_extra_int=1, my_extra_object={"keyOne": "valueOne", "keyTwo": 2}'
    )



from typing import Mapping

import pytest

from sag_py_logging.log_config_processors import FormatProcessor, JinjaProcessor


@pytest.fixture()
def format_template() -> str:
    return """{
        "version": 1,
        "disable_existing_loggers": "true",
        "root": {
            "handlers": ["myhandler"],
            "level": "INFO"
        },
        "handlers": {
            "myhandler": {
                "host": "${host}",
                "port": ${port},
                "formatter": "handler_formatter"
            }
        }
    }"""


@pytest.fixture()
def jinja_template() -> str:
    return """{
        "version": 1,
        "disable_existing_loggers": "true",
        "root": {
            "handlers": ["myhandler"],
            "level": "INFO"
        },
        "handlers": {
            "myhandler": {
                "host": "{{host}}",
                "port": {{port}},
                "formatter": "handler_formatter"
            }
        }
    }"""


def test__format_processor(format_template: str) -> None:
    # Arrange
    template_container: Mapping[str, object] = {"host": "myInsertedHost", "port": 99999}
    format_processor = FormatProcessor(template_container)

    # Act
    actual = format_processor(format_template)

    # Assert
    assert '"host": "myInsertedHost"' in actual
    assert '"port": 99999' in actual


def test__jinja_processor(jinja_template: str) -> None:
    # Arrange
    template_container: Mapping[str, object] = {"host": "myInsertedHost", "port": 99999}
    jinja_processor = JinjaProcessor(template_container)

    # Act
    actual = jinja_processor(jinja_template)

    # Assert
    assert '"host": "myInsertedHost"' in actual
    assert '"port": 99999' in actual


def test__format_processor__with_missing_key(format_template: str) -> None:
    with pytest.raises(KeyError) as exception:
        # Arrange
        template_container: Mapping[str, object] = {"port": 99999}
        format_processor = FormatProcessor(template_container)

        # Act
        format_processor(format_template)

    # Assert
    assert str(exception) == "<ExceptionInfo KeyError('host') tblen=4>"


def test__jinja_processor__with_missing_key(jinja_template: str) -> None:
    # Arrange
    template_container: Mapping[str, object] = {"port": 99999}
    jinja_processor = JinjaProcessor(template_container)

    # Act
    actual = jinja_processor(jinja_template)

    # Assert
    assert '"host": ""' in actual
    assert '"port": 99999' in actual



from abc import abstractmethod
from typing import Any, Dict


class LogLoader:
    @abstractmethod
    def __call__(self, log_template: str) -> Dict[str, Any]:
        pass  # pragma: no cover


class JsonLoader(LogLoader):
    def __init__(self) -> None:
        import json

        self.json = json

    def __call__(self, string_config: str) -> Dict[str, Any]:
        return self.json.loads(string_config)


class TomlLoader(LogLoader):
    def __init__(self) -> None:
        try:
            import tomli
        except ImportError as e:
            raise ModuleNotFoundError(
                "Module 'tomli' not installed.  Please run " "'python -m pip install sag-py-logging[toml]'"
            ) from e
        import tomli

        self.tomli = tomli

    def __call__(self, string_config: str) -> Dict[str, Any]:
        return self.tomli.loads(string_config)



from abc import abstractmethod
from typing import Mapping


class LogProcessor:
    @abstractmethod
    def __call__(self, log_template: str) -> str:
        pass  # pragma: no cover


class FormatProcessor(LogProcessor):
    def __init__(self, placeholder_container: Mapping[str, object]) -> None:
        from string import Template

        self.template = Template
        self.placeholder_container: Mapping[str, object] = placeholder_container

    def __call__(self, log_template: str) -> str:
        return self.template(log_template).substitute(self.placeholder_container)


class JinjaProcessor(LogProcessor):
    def __init__(self, placeholder_container: Mapping[str, object]) -> None:
        try:
            import jinja2
        except ImportError as e:
            raise ModuleNotFoundError(
                "Module 'jinja2' not installed.  Please run " "'python -m pip install sag-py-logging[jinja]'"
            ) from e

        self.jinja = jinja2
        self.placeholder_container: Mapping[str, object] = placeholder_container

    def __call__(self, log_template: str) -> str:
        template = self.jinja.Template(log_template)
        return template.render(self.placeholder_container)



import json
import logging
from typing import Any, Dict, List, Set


class ConsoleExtraFieldFilter(logging.Filter):
    def __init__(self, name: str = "") -> None:
        super().__init__(name=name)
        self.excluded_standard_fields: Set[str] = {
            "args",
            "asctime",
            "created",
            "exc_info",
            "exc_text",
            "filename",
            "funcName",
            "id",
            "levelname",
            "levelno",
            "lineno",
            "message",
            "module",
            "msecs",
            "msg",
            "name",
            "pathname",
            "process",
            "processName",
            "relativeCreated",
            "stack_info",
            "thread",
            "threadName",
            "logsource",
            "program",
            "type",
            "tags",
            "@metadata",
            "color_message",
        }

    def filter(self, record: logging.LogRecord) -> bool:
        extra_fields: Dict[str, Any] = self._get_extra_fields(record)
        extra_list: List[str] = self._to_key_value_strings(extra_fields)
        record.stringified_extra = ", ".join(extra_list)
        return True

    def _get_extra_fields(self, record: logging.LogRecord) -> Dict[str, Any]:
        return {key: value for key, value in record.__dict__.items() if key not in self.excluded_standard_fields}

    def _to_key_value_strings(self, extra_fields: Dict[str, Any]) -> List[str]:
        return [f"{key}={json.dumps(value)}" for key, value in extra_fields.items()]



import logging
import logging.config
from typing import Any, Dict, List, Optional

from sag_py_logging.log_config_loader import LogLoader
from sag_py_logging.log_config_processors import LogProcessor


def init_logging(
    config_file: str,
    loader: LogLoader,
    encoding: str = "UTF-8",
    processors: Optional[List[LogProcessor]] = None,
) -> Dict[str, Any]:
    config_template: str = _get_config_file_content(config_file, encoding)
    parsed_template: str = _parse_template(processors, config_template)
    log_config: Dict[str, Any] = loader(parsed_template)
    _init_python_logging(log_config)
    return log_config


def _get_config_file_content(config_file: str, encoding: str) -> str:
    with open(config_file, "r", encoding=encoding) as log_config_reader:
        return log_config_reader.read()


def _parse_template(processors: Optional[List[LogProcessor]], config_template: str) -> str:
    parsed_template: str = config_template
    if processors:
        for processor in processors:
            parsed_template = processor(parsed_template)
    return parsed_template


def _init_python_logging(log_config_dict: Dict[str, Any]) -> None:
    logging.basicConfig()
    logging.config.dictConfig(log_config_dict)



from typing import Any, Dict

from pytest import MonkeyPatch

from sag_py_logging.log_config_initializer import init_logging
from sag_py_logging.log_config_loader import LogLoader
from sag_py_logging.log_config_processors import LogProcessor


class TestLogProcessorOne(LogProcessor):
    def __call__(self, log_template: str) -> str:
        return f"{log_template}-LogProcessorOne"


class TestLogProcessorTwo(LogProcessor):
    def __call__(self, log_template: str) -> str:
        return f"{log_template}-LogProcessorTwo"


class TestLogLoader(LogLoader):
    def __call__(self, string_config: str) -> Dict[str, Any]:
        return {"result": string_config}


def _get_config_file_content_mock(config_file: str, encoding: str) -> str:
    return "myFileContent"


def _init_python_logging_mock(log_config_dict: Dict[str, Any]) -> None:
    pass


def test__init_logging(monkeypatch: MonkeyPatch) -> None:
    # Arrange
    monkeypatch.setattr("sag_py_logging.log_config_initializer._get_config_file_content", _get_config_file_content_mock)
    monkeypatch.setattr("sag_py_logging.log_config_initializer._init_python_logging", _init_python_logging_mock)

    # Act
    actual: Dict[str, Any] = init_logging(
        "myconfig.json", TestLogLoader(), processors=[TestLogProcessorOne(), TestLogProcessorTwo()]
    )

    # Assert
    assert actual["result"] == "myFileContent-LogProcessorOne-LogProcessorTwo"


def test__init_logging__without_processors(monkeypatch: MonkeyPatch) -> None:
    # Arrange
    monkeypatch.setattr("sag_py_logging.log_config_initializer._get_config_file_content", _get_config_file_content_mock)
    monkeypatch.setattr("sag_py_logging.log_config_initializer._init_python_logging", _init_python_logging_mock)

    # Act
    actual: Dict[str, Any] = init_logging("myconfig.json", TestLogLoader())

    # Assert
    assert actual["result"] == "myFileContent"



import setuptools

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

with open("requirements.txt", "r") as fin:
    REQS = fin.read().splitlines()

setuptools.setup(
    name="sag-py-logging",
    version="0.3.0",
    description="Initialize logging from a configuration json",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/SamhammerAG/sag_py_logging",
    author="Samhammer AG",
    author_email="support@samhammer.de",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development",
    ],
    keywords="logging, extra, fields",
    packages=setuptools.find_packages(exclude=["tests"]),
    package_data={"sag_py_logging": ["py.typed"]},
    python_requires=">=3.8",
    install_requires=REQS,
    extras_require={"dev": ["pytest"], "jinja": ["Jinja2>=3.1.2"], "toml": ["tomli>=2.0.1"]},
    project_urls={
        "Documentation": "https://github.com/SamhammerAG/sag_py_logging",
        "Bug Reports": "https://github.com/SamhammerAG/sag_py_logging/issues",
        "Source": "https://github.com/SamhammerAG/sag_py_logging",
    },
)



import pytest

from sag_py_logging.log_config_loader import JsonLoader, TomlLoader


@pytest.fixture()
def test_json() -> str:
    return """{
        "version": 1,
        "disable_existing_loggers": true,
        "root": {
            "handlers": ["myhandler"],
            "level": "INFO"
        },
        "handlers": {
            "myhandler": {
                "formatter": "handler_formatter"
            }
        }
    }"""


@pytest.fixture()
def test_toml() -> str:
    return """version = 1
        disable_existing_loggers = true

        [root]
        handlers = [ "myhandler" ]
        level = "INFO"

        [handlers.myhandler]
        formatter = "handler_formatter"
    """


def test__json_loader(test_json: str) -> None:
    # Arrange
    json_loader = JsonLoader()

    # Act
    actual = json_loader(test_json)

    # Assert
    assert actual["version"] == 1
    assert actual["disable_existing_loggers"] == True
    assert actual["root"]["handlers"][0] == "myhandler"
    assert actual["root"]["level"] == "INFO"
    assert actual["handlers"]["myhandler"]["formatter"] == "handler_formatter"


def test__toml_loader(test_toml: str) -> None:
    # Arrange
    toml_loader = TomlLoader()

    # Act
    actual = toml_loader(test_toml)

    # Assert
    assert actual["version"] == 1
    assert actual["disable_existing_loggers"] == True
    assert actual["root"]["handlers"][0] == "myhandler"
    assert actual["root"]["level"] == "INFO"
    assert actual["handlers"]["myhandler"]["formatter"] == "handler_formatter"
